/*
 * Copyright 2016 - 2021 gary@drinkingtea.net
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#pragma once

#include <ox/fs/fs.hpp>

#include "assetmanager.hpp"

namespace nostalgia::core {

class Context;

class Drawer {
	public:
		virtual ~Drawer() = default;
		virtual void draw(Context*) noexcept = 0;
};

// User Input Output
class Context {
	public:
		ox::UniquePtr<ox::FileSystem> rom;
		ox::Vector<Drawer*, 5> drawers;
		const char *appName = "Nostalgia";
#ifndef OX_BARE_METAL
		AssetManager assetManager;
#endif
	private:
		void *m_customData = nullptr;
		void *m_windowerData = nullptr;
		void *m_rendererData = nullptr;

	public:
		Context() noexcept = default;

		Context(Context &other) noexcept = delete;
		Context(const Context &other) noexcept = delete;
		Context(const Context &&other) noexcept = delete;

		constexpr void setCustomData(void *customData) noexcept {
			m_customData = customData;
		}

		template<typename T>
		[[nodiscard]]
		constexpr T *customData() noexcept {
			return static_cast<T*>(m_customData);
		}

		constexpr void setWindowerData(void *windowerData) noexcept {
			m_windowerData = windowerData;
		}

		template<typename T>
		[[nodiscard]]
		constexpr T *windowerData() noexcept {
			return static_cast<T*>(m_windowerData);
		}

		constexpr void setRendererData(void *rendererData) noexcept {
			m_rendererData = rendererData;
		}

		template<typename T>
		[[nodiscard]]
		constexpr T *rendererData() noexcept {
			return static_cast<T*>(m_rendererData);
		}

};

}

